@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -24,6 +24,7 @@ import org.apache.catalina.Executor;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.util.LifecycleMBeanBase;
+import org.apache.tomcat.util.res.StringManager;
 import org.apache.tomcat.util.threads.ResizableExecutor;
 import org.apache.tomcat.util.threads.TaskQueue;
 import org.apache.tomcat.util.threads.TaskThreadFactory;
@@ -31,7 +32,10 @@ import org.apache.tomcat.util.threads.ThreadPoolExecutor;
 
 public class StandardThreadExecutor extends LifecycleMBeanBase
         implements Executor, ResizableExecutor {
-    
+
+    protected static final StringManager sm =
+            StringManager.getManager(Constants.Package);
+
     // ---------------------------------------------- Properties
     /**
      * Default thread priority
@@ -42,37 +46,37 @@ public class StandardThreadExecutor extends LifecycleMBeanBase
      * Run threads in daemon or non-daemon state
      */
     protected boolean daemon = true;
-    
+
     /**
      * Default name prefix for the thread name
      */
     protected String namePrefix = "tomcat-exec-";
-    
+
     /**
      * max number of threads
      */
     protected int maxThreads = 200;
-    
+
     /**
      * min number of threads
      */
     protected int minSpareThreads = 25;
-    
+
     /**
      * idle time in milliseconds
      */
     protected int maxIdleTime = 60000;
-    
+
     /**
      * The executor we use for this component
      */
     protected ThreadPoolExecutor executor = null;
-    
+
     /**
      * the name of this thread pool
      */
     protected String name;
-    
+
     /**
      * prestart threads?
      */
@@ -82,15 +86,15 @@ public class StandardThreadExecutor extends LifecycleMBeanBase
      * The maximum number of elements that can queue up before we reject them
      */
     protected int maxQueueSize = Integer.MAX_VALUE;
-    
+
     /**
      * After a context is stopped, threads in the pool are renewed. To avoid
      * renewing all threads at the same time, this delay is observed between 2
      * threads being renewed.
      */
-    protected long threadRenewalDelay = 
+    protected long threadRenewalDelay =
         org.apache.tomcat.util.threads.Constants.DEFAULT_THREAD_RENEWAL_DELAY;
-    
+
     private TaskQueue taskqueue = null;
     // ---------------------------------------------- Constructors
     public StandardThreadExecutor() {
@@ -99,13 +103,13 @@ public class StandardThreadExecutor extends LifecycleMBeanBase
 
 
     // ---------------------------------------------- Public Methods
-    
+
     @Override
     protected void initInternal() throws LifecycleException {
         super.initInternal();
     }
 
-    
+
     /**
      * Start the component and implement the requirements
      * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
@@ -119,6 +123,7 @@ public class StandardThreadExecutor extends LifecycleMBeanBase
         taskqueue = new TaskQueue(maxQueueSize);
         TaskThreadFactory tf = new TaskThreadFactory(namePrefix,daemon,getThreadPriority());
         executor = new ThreadPoolExecutor(getMinSpareThreads(), getMaxThreads(), maxIdleTime, TimeUnit.MILLISECONDS,taskqueue, tf);
+        executor.setThreadRenewalDelay(threadRenewalDelay);
         if (prestartminSpareThreads) {
             executor.prestartAllCoreThreads();
         }
@@ -139,40 +144,46 @@ public class StandardThreadExecutor extends LifecycleMBeanBase
     protected void stopInternal() throws LifecycleException {
 
         setState(LifecycleState.STOPPING);
-        if ( executor != null ) executor.shutdownNow();
+        if (executor != null) {
+            executor.shutdownNow();
+        }
         executor = null;
         taskqueue = null;
     }
 
-    
+
     @Override
     protected void destroyInternal() throws LifecycleException {
         super.destroyInternal();
     }
 
-    
+
     @Override
     public void execute(Runnable command, long timeout, TimeUnit unit) {
-        if ( executor != null ) {
+        if (executor != null) {
             executor.execute(command,timeout,unit);
-        } else { 
-            throw new IllegalStateException("StandardThreadExecutor not started.");
+        } else {
+            throw new IllegalStateException(sm.getString("standardThreadExecutor.notStarted"));
         }
     }
-    
-    
+
+
     @Override
     public void execute(Runnable command) {
-        if ( executor != null ) {
+        if (executor != null) {
             try {
                 executor.execute(command);
             } catch (RejectedExecutionException rx) {
                 //there could have been contention around the queue
-                if ( !( (TaskQueue) executor.getQueue()).force(command) ) throw new RejectedExecutionException("Work queue full.");
+                if (!((TaskQueue) executor.getQueue()).force(command)) {
+                    throw new RejectedExecutionException(sm.getString("standardThreadExecutor.queueFull"));
+                }
             }
-        } else throw new IllegalStateException("StandardThreadPool not started.");
+        } else {
+            throw new IllegalStateException(sm.getString("standardThreadExecutor.notStarted"));
+        }
     }
-    
+
     public void contextStopping() {
         if (executor != null) {
             executor.contextStopping();
@@ -254,15 +265,15 @@ public class StandardThreadExecutor extends LifecycleMBeanBase
     public void setName(String name) {
         this.name = name;
     }
-    
+
     public void setMaxQueueSize(int size) {
         this.maxQueueSize = size;
     }
-    
+
     public int getMaxQueueSize() {
         return maxQueueSize;
     }
-    
+
     public long getThreadRenewalDelay() {
         return threadRenewalDelay;
     }
@@ -317,7 +328,7 @@ public class StandardThreadExecutor extends LifecycleMBeanBase
     public boolean resizeQueue(int capacity) {
         return false;
     }
-    
+
 
     @Override
     protected String getDomainInternal() {
