@@ -24,17 +24,16 @@ import org.apache.catalina.tribes.io.ObjectReader;
 import org.apache.catalina.tribes.transport.AbstractRxTask;
 import org.apache.catalina.tribes.transport.ReceiverBase;
 import org.apache.catalina.tribes.transport.RxTaskPool;
+import org.apache.catalina.tribes.util.StringManager;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 
-/**
- *
- * @author Filip Hanik
- */
 public class BioReceiver extends ReceiverBase implements Runnable {
 
     private static final Log log = LogFactory.getLog(BioReceiver.class);
 
+    protected static final StringManager sm = StringManager.getManager(BioReceiver.class);
+
     protected ServerSocket serverSocket;
 
     public BioReceiver() {
@@ -47,28 +46,30 @@ public class BioReceiver extends ReceiverBase implements Runnable {
         try {
             setPool(new RxTaskPool(getMaxThreads(),getMinThreads(),this));
         } catch (Exception x) {
-            log.fatal("ThreadPool can initilzed. Listener not started", x);
+            log.fatal(sm.getString("bioReceiver.threadpool.fail"), x);
             if ( x instanceof IOException ) throw (IOException)x;
             else throw new IOException(x.getMessage());
         }
         try {
             getBind();
             bind();
-            Thread t = new Thread(this, "BioReceiver");
+            String channelName = "";
+            if (getChannel().getName() != null) channelName = "[" + getChannel().getName() + "]";
+            Thread t = new Thread(this, "BioReceiver" + channelName);
             t.setDaemon(true);
             t.start();
         } catch (Exception x) {
-            log.fatal("Unable to start cluster receiver", x);
+            log.fatal(sm.getString("bioReceiver.start.fail"), x);
             if ( x instanceof IOException ) throw (IOException)x;
             else throw new IOException(x.getMessage());
         }
     }
-    
+
     @Override
     public AbstractRxTask createRxTask() {
         return getReplicationThread();
     }
-    
+
     protected BioReplicationTask getReplicationThread() {
         BioReplicationTask result = new BioReplicationTask(this);
         result.setOptions(getWorkerThreadOptions());
@@ -83,7 +84,7 @@ public class BioReceiver extends ReceiverBase implements Runnable {
             this.serverSocket.close();
         } catch (Exception x) {
             if (log.isDebugEnabled()) {
-                log.debug("Failed to close socket", x);
+                log.debug(sm.getString("bioReceiver.socket.closeFailed"), x);
             }
         }
         super.stop();
@@ -104,13 +105,13 @@ public class BioReceiver extends ReceiverBase implements Runnable {
         try {
             listen();
         } catch (Exception x) {
-            log.error("Unable to run replication listener.", x);
+            log.error(sm.getString("bioReceiver.run.fail"), x);
         }
     }
-    
+
     public void listen() throws Exception {
         if (doListen()) {
-            log.warn("ServerSocket already started");
+            log.warn(sm.getString("bioReceiver.already.started"));
             return;
         }
         setListen(true);
@@ -119,7 +120,7 @@ public class BioReceiver extends ReceiverBase implements Runnable {
             Socket socket = null;
             if ( getTaskPool().available() < 1 ) {
                 if ( log.isWarnEnabled() )
-                    log.warn("All BIO server replication threads are busy, unable to handle more requests until a thread is freed up.");
+                    log.warn(sm.getString("bioReceiver.threads.busy"));
             }
             BioReplicationTask task = (BioReplicationTask)getTaskPool().getRxTask();
             if ( task == null ) continue; //should never happen
@@ -129,9 +130,9 @@ public class BioReceiver extends ReceiverBase implements Runnable {
                 if ( doListen() ) throw x;
             }
             if ( !doListen() ) {
-                task.setDoRun(false);
                 task.serviceSocket(null,null);
                 getExecutor().execute(task);
+                task.close();
                 break; //regular shutdown
             }
             if ( socket == null ) continue;
@@ -148,6 +149,6 @@ public class BioReceiver extends ReceiverBase implements Runnable {
             getExecutor().execute(task);
         }//while
     }
-    
+
 
 }
\ No newline at end of file
