@@ -5,16 +5,15 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.jasper.compiler;
 
 import java.io.BufferedOutputStream;
@@ -30,6 +29,7 @@ import java.io.InputStreamReader;
 import java.io.Reader;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.StringTokenizer;
@@ -54,40 +54,40 @@ import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
 
 /**
  * JDT class compiler. This compiler will load source dependencies from the
- * context classloader, reducing dramatically disk access during 
+ * context classloader, reducing dramatically disk access during
  * the compilation process.
  *
- * @author Cocoon2
+ * Based on code from Cocoon2.
+ *
  * @author Remy Maucherat
  */
 public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
 
     private final Log log = LogFactory.getLog(JDTCompiler.class); // must not be static
-   
-    /** 
+
+    /**
      * Compile the servlet from .java file to .class file
      */
     @Override
-    protected void generateClass(String[] smap)
+    protected void generateClass(Map<String,SmapStratum> smaps)
         throws FileNotFoundException, JasperException, Exception {
 
         long t1 = 0;
         if (log.isDebugEnabled()) {
             t1 = System.currentTimeMillis();
         }
-        
+
         final String sourceFile = ctxt.getServletJavaFileName();
         final String outputDir = ctxt.getOptions().getScratchDir().getAbsolutePath();
         String packageName = ctxt.getServletPackageName();
-        final String targetClassName = 
-            ((packageName.length() != 0) ? (packageName + ".") : "") 
+        final String targetClassName =
+            ((packageName.length() != 0) ? (packageName + ".") : "")
                     + ctxt.getServletClassName();
         final ClassLoader classLoader = ctxt.getJspLoader();
         String[] fileNames = new String[] {sourceFile};
         String[] classNames = new String[] {targetClassName};
-        final ArrayList<JavacErrorDetail> problemList =
-            new ArrayList<JavacErrorDetail>();
-        
+        final List<JavacErrorDetail> problemList = new ArrayList<>();
+
         class CompilationUnit implements ICompilationUnit {
 
             private final String className;
@@ -102,49 +102,29 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
             public char[] getFileName() {
                 return sourceFile.toCharArray();
             }
-            
+
             @Override
             public char[] getContents() {
                 char[] result = null;
-                FileInputStream is = null;
-                InputStreamReader isr = null;
-                Reader reader = null;
-                try {
-                    is = new FileInputStream(sourceFile);
-                    isr = new InputStreamReader(is,
-                            ctxt.getOptions().getJavaEncoding());
-                    reader = new BufferedReader(isr);
+                try (FileInputStream is = new FileInputStream(sourceFile);
+                        InputStreamReader isr = new InputStreamReader(
+                                is, ctxt.getOptions().getJavaEncoding());
+                        Reader reader = new BufferedReader(isr)) {
                     char[] chars = new char[8192];
                     StringBuilder buf = new StringBuilder();
                     int count;
-                    while ((count = reader.read(chars, 0, 
+                    while ((count = reader.read(chars, 0,
                                                 chars.length)) > 0) {
                         buf.append(chars, 0, count);
                     }
                     result = new char[buf.length()];
                     buf.getChars(0, result.length, result, 0);
                 } catch (IOException e) {
-                    log.error("Compilation error", e);
-                } finally {
-                    if (reader != null) {
-                        try {
-                            reader.close();
-                        } catch (IOException ioe) {/*Ignore*/}
-                    }
-                    if (isr != null) {
-                        try {
-                            isr.close();
-                        } catch (IOException ioe) {/*Ignore*/}
-                    }
-                    if (is != null) {
-                        try {
-                            is.close();
-                        } catch (IOException exc) {/*Ignore*/}
-                    }
+                    log.error(Localizer.getMessage("jsp.error.compilation.source", sourceFile), e);
                 }
                 return result;
             }
-            
+
             @Override
             public char[] getMainTypeName() {
                 int dot = className.lastIndexOf('.');
@@ -153,10 +133,10 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                 }
                 return className.toCharArray();
             }
-            
+
             @Override
             public char[][] getPackageName() {
-                StringTokenizer izer = 
+                StringTokenizer izer =
                     new StringTokenizer(className, ".");
                 char[][] result = new char[izer.countTokens()-1][];
                 for (int i = 0; i < result.length; i++) {
@@ -175,51 +155,51 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
         final INameEnvironment env = new INameEnvironment() {
 
                 @Override
-                public NameEnvironmentAnswer 
+                public NameEnvironmentAnswer
                     findType(char[][] compoundTypeName) {
-                    String result = "";
-                    String sep = "";
+                    StringBuilder result = new StringBuilder();
                     for (int i = 0; i < compoundTypeName.length; i++) {
-                        result += sep;
-                        result += new String(compoundTypeName[i]);
-                        sep = ".";
+                        if(i > 0)
+                            result.append('.');
+                        result.append(compoundTypeName[i]);
                     }
-                    return findType(result);
+                    return findType(result.toString());
                 }
 
                 @Override
-                public NameEnvironmentAnswer 
-                    findType(char[] typeName, 
+                public NameEnvironmentAnswer
+                    findType(char[] typeName,
                              char[][] packageName) {
-                        String result = "";
-                        String sep = "";
-                        for (int i = 0; i < packageName.length; i++) {
-                            result += sep;
-                            result += new String(packageName[i]);
-                            sep = ".";
+                        StringBuilder result = new StringBuilder();
+                        int i=0;
+                        for (; i < packageName.length; i++) {
+                            if(i > 0)
+                                result.append('.');
+                            result.append(packageName[i]);
                         }
-                        result += sep;
-                        result += new String(typeName);
-                        return findType(result);
+                        if(i > 0)
+                            result.append('.');
+                        result.append(typeName);
+                        return findType(result.toString());
                 }
-                
+
                 private NameEnvironmentAnswer findType(String className) {
 
-                    InputStream is = null;
-                    try {
-                        if (className.equals(targetClassName)) {
-                            ICompilationUnit compilationUnit = 
-                                new CompilationUnit(sourceFile, className);
-                            return 
-                                new NameEnvironmentAnswer(compilationUnit, null);
-                        }
-                        String resourceName = 
+                    if (className.equals(targetClassName)) {
+                        ICompilationUnit compilationUnit =
+                            new CompilationUnit(sourceFile, className);
+                        return
+                            new NameEnvironmentAnswer(compilationUnit, null);
+                    }
+
+                    String resourceName =
                             className.replace('.', '/') + ".class";
-                        is = classLoader.getResourceAsStream(resourceName);
+
+                    try (InputStream is = classLoader.getResourceAsStream(resourceName)) {
                         if (is != null) {
                             byte[] classBytes;
                             byte[] buf = new byte[8192];
-                            ByteArrayOutputStream baos = 
+                            ByteArrayOutputStream baos =
                                 new ByteArrayOutputStream(buf.length);
                             int count;
                             while ((count = is.read(buf, 0, buf.length)) > 0) {
@@ -228,24 +208,16 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                             baos.flush();
                             classBytes = baos.toByteArray();
                             char[] fileName = className.toCharArray();
-                            ClassFileReader classFileReader = 
-                                new ClassFileReader(classBytes, fileName, 
+                            ClassFileReader classFileReader =
+                                new ClassFileReader(classBytes, fileName,
                                                     true);
-                            return 
+                            return
                                 new NameEnvironmentAnswer(classFileReader, null);
                         }
                     } catch (IOException exc) {
-                        log.error("Compilation error", exc);
+                        log.error(Localizer.getMessage("jsp.error.compilation.dependent", className), exc);
                     } catch (org.eclipse.jdt.internal.compiler.classfmt.ClassFormatException exc) {
-                        log.error("Compilation error", exc);
-                    } finally {
-                        if (is != null) {
-                            try {
-                                is.close();
-                            } catch (IOException exc) {
-                                // Ignore
-                            }
-                        }
+                        log.error(Localizer.getMessage("jsp.error.compilation.dependent", className), exc);
                     }
                     return null;
                 }
@@ -255,33 +227,38 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                         return false;
                     }
                     String resourceName = result.replace('.', '/') + ".class";
-                    InputStream is = 
-                        classLoader.getResourceAsStream(resourceName);
-                    return is == null;
+                    try (InputStream is =
+                        classLoader.getResourceAsStream(resourceName)) {
+                        return is == null;
+                    } catch (IOException e) {
+                        // we are here, since close on is failed. That means it was not null
+                        return false;
+                    }
                 }
 
                 @Override
-                public boolean isPackage(char[][] parentPackageName, 
+                public boolean isPackage(char[][] parentPackageName,
                                          char[] packageName) {
-                    String result = "";
-                    String sep = "";
+                    StringBuilder result = new StringBuilder();
+                    int i=0;
                     if (parentPackageName != null) {
-                        for (int i = 0; i < parentPackageName.length; i++) {
-                            result += sep;
-                            String str = new String(parentPackageName[i]);
-                            result += str;
-                            sep = ".";
+                        for (; i < parentPackageName.length; i++) {
+                            if(i > 0)
+                                result.append('.');
+                            result.append(parentPackageName[i]);
                         }
                     }
-                    String str = new String(packageName);
-                    if (Character.isUpperCase(str.charAt(0))) {
-                        if (!isPackage(result)) {
+
+                    if (Character.isUpperCase(packageName[0])) {
+                        if (!isPackage(result.toString())) {
                             return false;
                         }
                     }
-                    result += sep;
-                    result += str;
-                    return isPackage(result);
+                    if(i > 0)
+                        result.append('.');
+                    result.append(packageName);
+
+                    return isPackage(result.toString());
                 }
 
                 @Override
@@ -290,10 +267,10 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
 
             };
 
-        final IErrorHandlingPolicy policy = 
+        final IErrorHandlingPolicy policy =
             DefaultErrorHandlingPolicies.proceedWithAllProblems();
 
-        final Map<String,String> settings = new HashMap<String,String>();
+        final Map<String,String> settings = new HashMap<>();
         settings.put(CompilerOptions.OPTION_LineNumberAttribute,
                      CompilerOptions.GENERATE);
         settings.put(CompilerOptions.OPTION_SourceFileAttribute,
@@ -318,7 +295,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
             } else if(opt.equals("1.2")) {
                 settings.put(CompilerOptions.OPTION_Source,
                              CompilerOptions.VERSION_1_2);
-            } else if(opt.equals("1.3")) { 
+            } else if(opt.equals("1.3")) {
                 settings.put(CompilerOptions.OPTION_Source,
                              CompilerOptions.VERSION_1_3);
             } else if(opt.equals("1.4")) {
@@ -333,17 +310,39 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
             } else if(opt.equals("1.7")) {
                 settings.put(CompilerOptions.OPTION_Source,
                              CompilerOptions.VERSION_1_7);
-            } else {
-                log.warn("Unknown source VM " + opt + " ignored.");
+            } else if(opt.equals("1.8")) {
                 settings.put(CompilerOptions.OPTION_Source,
-                        CompilerOptions.VERSION_1_5);
+                             CompilerOptions.VERSION_1_8);
+            // Version format changed from Java 9 onwards.
+            // Support old format that was used in EA implementation as well
+            } else if(opt.equals("9") || opt.equals("1.9")) {
+                settings.put(CompilerOptions.OPTION_Source,
+                             CompilerOptions.VERSION_9);
+            } else if(opt.equals("10")) {
+                settings.put(CompilerOptions.OPTION_Source,
+                             CompilerOptions.VERSION_10);
+            } else if(opt.equals("11")) {
+                settings.put(CompilerOptions.OPTION_Source,
+                             CompilerOptions.VERSION_11);
+            } else if(opt.equals("12")) {
+                settings.put(CompilerOptions.OPTION_Source,
+                             CompilerOptions.VERSION_12);
+            } else if(opt.equals("13")) {
+                // Constant not available in latest ECJ version shipped with
+                // Tomcat. May be supported in a snapshot build.
+                // This is checked against the actual version below.
+                settings.put(CompilerOptions.OPTION_Source, "13");
+            } else {
+                log.warn(Localizer.getMessage("jsp.warning.unknown.sourceVM", opt));
+                settings.put(CompilerOptions.OPTION_Source,
+                        CompilerOptions.VERSION_1_8);
             }
         } else {
-            // Default to 1.5
+            // Default to 1.8
             settings.put(CompilerOptions.OPTION_Source,
-                    CompilerOptions.VERSION_1_5);
+                    CompilerOptions.VERSION_1_8);
         }
-        
+
         // Target JVM
         if(ctxt.getOptions().getCompilerTargetVM() != null) {
             String opt = ctxt.getOptions().getCompilerTargetVM();
@@ -353,7 +352,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
             } else if(opt.equals("1.2")) {
                 settings.put(CompilerOptions.OPTION_TargetPlatform,
                              CompilerOptions.VERSION_1_2);
-            } else if(opt.equals("1.3")) { 
+            } else if(opt.equals("1.3")) {
                 settings.put(CompilerOptions.OPTION_TargetPlatform,
                              CompilerOptions.VERSION_1_3);
             } else if(opt.equals("1.4")) {
@@ -374,22 +373,55 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                              CompilerOptions.VERSION_1_7);
                 settings.put(CompilerOptions.OPTION_Compliance,
                         CompilerOptions.VERSION_1_7);
-            } else {
-                log.warn("Unknown target VM " + opt + " ignored.");
+            } else if(opt.equals("1.8")) {
                 settings.put(CompilerOptions.OPTION_TargetPlatform,
-                        CompilerOptions.VERSION_1_5);
+                             CompilerOptions.VERSION_1_8);
+                settings.put(CompilerOptions.OPTION_Compliance,
+                        CompilerOptions.VERSION_1_8);
+            // Version format changed from Java 9 onwards.
+            // Support old format that was used in EA implementation as well
+            } else if(opt.equals("9") || opt.equals("1.9")) {
+                settings.put(CompilerOptions.OPTION_TargetPlatform,
+                             CompilerOptions.VERSION_9);
+                settings.put(CompilerOptions.OPTION_Compliance,
+                        CompilerOptions.VERSION_9);
+            } else if(opt.equals("10")) {
+                settings.put(CompilerOptions.OPTION_TargetPlatform,
+                        CompilerOptions.VERSION_10);
+                settings.put(CompilerOptions.OPTION_Compliance,
+                        CompilerOptions.VERSION_10);
+            } else if(opt.equals("11")) {
+                settings.put(CompilerOptions.OPTION_TargetPlatform,
+                        CompilerOptions.VERSION_11);
+                settings.put(CompilerOptions.OPTION_Compliance,
+                        CompilerOptions.VERSION_11);
+            } else if(opt.equals("12")) {
+                settings.put(CompilerOptions.OPTION_TargetPlatform,
+                        CompilerOptions.VERSION_12);
+                settings.put(CompilerOptions.OPTION_Compliance,
+                        CompilerOptions.VERSION_12);
+            } else if(opt.equals("13")) {
+                // Constant not available in latest ECJ version shipped with
+                // Tomcat. May be supported in a snapshot build.
+                // This is checked against the actual version below.
+                settings.put(CompilerOptions.OPTION_TargetPlatform, "13");
+                settings.put(CompilerOptions.OPTION_Compliance, "13");
+            } else {
+                log.warn(Localizer.getMessage("jsp.warning.unknown.targetVM", opt));
+                settings.put(CompilerOptions.OPTION_TargetPlatform,
+                        CompilerOptions.VERSION_1_8);
             }
         } else {
-            // Default to 1.5
+            // Default to 1.8
             settings.put(CompilerOptions.OPTION_TargetPlatform,
-                    CompilerOptions.VERSION_1_5);
+                    CompilerOptions.VERSION_1_8);
             settings.put(CompilerOptions.OPTION_Compliance,
-                    CompilerOptions.VERSION_1_5);
+                    CompilerOptions.VERSION_1_8);
         }
 
-        final IProblemFactory problemFactory = 
+        final IProblemFactory problemFactory =
             new DefaultProblemFactory(Locale.getDefault());
-        
+
         final ICompilerRequestor requestor = new ICompilerRequestor() {
                 @Override
                 public void acceptResult(CompilationResult result) {
@@ -399,14 +431,14 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                             for (int i = 0; i < problems.length; i++) {
                                 IProblem problem = problems[i];
                                 if (problem.isError()) {
-                                    String name = 
+                                    String name =
                                         new String(problems[i].getOriginatingFileName());
                                     try {
                                         problemList.add(ErrorDispatcher.createJavacError
-                                                (name, pageNodes, new StringBuilder(problem.getMessage()), 
+                                                (name, pageNodes, new StringBuilder(problem.getMessage()),
                                                         problem.getSourceLineNumber(), ctxt));
                                     } catch (JasperException e) {
-                                        log.error("Error visiting node", e);
+                                        log.error(Localizer.getMessage("jsp.error.compilation.jdtProblemError"), e);
                                     }
                                 }
                             }
@@ -415,40 +447,56 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
                             ClassFile[] classFiles = result.getClassFiles();
                             for (int i = 0; i < classFiles.length; i++) {
                                 ClassFile classFile = classFiles[i];
-                                char[][] compoundName = 
+                                char[][] compoundName =
                                     classFile.getCompoundName();
-                                String className = "";
-                                String sep = "";
-                                for (int j = 0; 
+                                StringBuilder classFileName = new StringBuilder(outputDir).append('/');
+                                for (int j = 0;
                                      j < compoundName.length; j++) {
-                                    className += sep;
-                                    className += new String(compoundName[j]);
-                                    sep = ".";
+                                    if(j > 0)
+                                        classFileName.append('/');
+                                    classFileName.append(compoundName[j]);
                                 }
                                 byte[] bytes = classFile.getBytes();
-                                String outFile = outputDir + "/" + 
-                                    className.replace('.', '/') + ".class";
-                                FileOutputStream fout = 
-                                    new FileOutputStream(outFile);
-                                BufferedOutputStream bos = 
-                                    new BufferedOutputStream(fout);
-                                bos.write(bytes);
-                                bos.close();
+                                classFileName.append(".class");
+                                try (FileOutputStream fout = new FileOutputStream(
+                                        classFileName.toString());
+                                        BufferedOutputStream bos = new BufferedOutputStream(fout)) {
+                                    bos.write(bytes);
+                                }
                             }
                         }
                     } catch (IOException exc) {
-                        log.error("Compilation error", exc);
+                        log.error(Localizer.getMessage("jsp.error.compilation.jdt"), exc);
                     }
                 }
             };
 
-        ICompilationUnit[] compilationUnits = 
+        ICompilationUnit[] compilationUnits =
             new ICompilationUnit[classNames.length];
         for (int i = 0; i < compilationUnits.length; i++) {
             String className = classNames[i];
             compilationUnits[i] = new CompilationUnit(fileNames[i], className);
         }
         CompilerOptions cOptions = new CompilerOptions(settings);
+
+        // Check source/target JDK versions as the newest versions are allowed
+        // in Tomcat configuration but may not be supported by the ECJ version
+        // being used.
+        String requestedSource = ctxt.getOptions().getCompilerSourceVM();
+        if (requestedSource != null) {
+            String actualSource = CompilerOptions.versionFromJdkLevel(cOptions.sourceLevel);
+            if (!requestedSource.equals(actualSource)) {
+                log.warn(Localizer.getMessage("jsp.warning.unsupported.sourceVM", requestedSource, actualSource));
+            }
+        }
+        String requestedTarget = ctxt.getOptions().getCompilerTargetVM();
+        if (requestedTarget != null) {
+            String actualTarget = CompilerOptions.versionFromJdkLevel(cOptions.targetJDK);
+            if (!requestedTarget.equals(actualTarget)) {
+                log.warn(Localizer.getMessage("jsp.warning.unsupported.targetVM", requestedTarget, actualTarget));
+            }
+        }
+
         cOptions.parseLiteralExpressionsAsConstants = true;
         Compiler compiler = new Compiler(env,
                                          policy,
@@ -459,15 +507,18 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
 
         if (!ctxt.keepGenerated()) {
             File javaFile = new File(ctxt.getServletJavaFileName());
-            javaFile.delete();
+            if (!javaFile.delete()) {
+                throw new JasperException(Localizer.getMessage(
+                        "jsp.warning.compiler.javafile.delete.fail", javaFile));
+            }
         }
-    
+
         if (!problemList.isEmpty()) {
-            JavacErrorDetail[] jeds = 
+            JavacErrorDetail[] jeds =
                 problemList.toArray(new JavacErrorDetail[0]);
             errDispatcher.javacError(jeds);
         }
-        
+
         if( log.isDebugEnabled() ) {
             long t2=System.currentTimeMillis();
             log.debug("Compiled " + ctxt.getServletJavaFileName() + " "
@@ -480,10 +531,7 @@ public class JDTCompiler extends org.apache.jasper.compiler.Compiler {
 
         // JSR45 Support
         if (! options.isSmapSuppressed()) {
-            SmapUtil.installSmap(smap);
+            SmapUtil.installSmap(smaps);
         }
-        
     }
-    
-    
 }
